----------------------------------------------------------------------------------
-- Company: UAM 
-- Design Name: bin_reader
-- Project Name: bin_reader
-- Description: 
--   Binary reader. Converts switches positions into display of the corresponding
--   decimal number
-- Dependencies: 
--   bin_reader_pkg
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.bin_reader_pkg.all; -- use project's package

entity bin_reader is
   Port  ( Clk : in  STD_LOGIC;
           ResetX : in  STD_LOGIC;
           Switches : in  STD_LOGIC_VECTOR (7 downto 0);
           Segments : out  STD_LOGIC_VECTOR (7 downto 0);
           Anodes : out  STD_LOGIC_VECTOR (3 downto 0)
         );
end bin_reader;

architecture Behavioral of bin_reader is

   component counter is
     generic (
              COUNT_LENGTH : integer := 100  -- Count 0 to (COUNT_LENGTH-1)
             );
     port (
           -- Inputs:
           Clk       : in std_logic;  -- Clock (rising edge)
           Reset     : in std_logic;  -- Async reset, active high
           SyncReset : in std_logic;  -- Sync reset, active high
           EnableIn  : in std_logic;  -- Clock enable. If 0 count is stopped
           -- Outputs:
           Count     : out std_logic_vector (log2_ceil(COUNT_LENGTH)-1 downto 0);
                                      -- Output count value
           EnableOut : out std_logic  -- '1' when Count is last value and EnableIn
                                      --   (combinational output)
          );
   end component;
  
   component hex2led is
       Port ( Hex : in  STD_LOGIC_VECTOR (3 downto 0);
              Led : out  STD_LOGIC_VECTOR (6 downto 0));
   end component;
  
   component display_ctrl is
     generic (
              REFRESH_NCYCLES : integer := 100000 -- Cycles between anode changes
             );
     port (
           Clk           : in  std_logic;  -- Clock (rising edge)
           Reset         : in  std_logic;  -- Async reset, active high
           Digit0        : in  std_logic_vector (6 downto 0); -- 1/100 s digit
           Digit1        : in  std_logic_vector (6 downto 0); -- 1/100 s digit
           Digit2        : in  std_logic_vector (6 downto 0); -- 1/10 s digit
           Segments      : out std_logic_vector (7 downto 0); -- Segment,active 0
                                                              --  (dp,g downto a)
           Anodes        : out std_logic_vector (3 downto 0) -- Anodes, active 0
          );        
   end component;
  
  
   component spram is 
     generic (
              SIZE : integer := 1024;
              ADDR_WIDTH : integer := 10;
              DATA_WIDTH : integer := 8
             );
     port (Clk      : in  std_logic;   
           We       : in  std_logic;   
           Address  : in  std_logic_vector(ADDR_WIDTH-1 downto 0);   
           DataIn   : in  std_logic_vector(DATA_WIDTH-1 downto 0);   
           DataOut  : out std_logic_vector(DATA_WIDTH-1 downto 0)
          );   
   end component;   
  
   component bin_reader_ctrl is
     port (
           Clk         : in  std_logic;
           Reset       : in  std_logic;
           Switches    : in  std_logic_vector (7 downto 0);
           MemWe       : out std_logic;
           MemAddress  : out std_logic_vector (7 downto 0);
           EnableCount : out std_logic
          );
   end component;
  
   component reset_adapt is
     generic (
               META_DEPTH : integer := 3  -- Number of flops in anti-metastability
                                          --   chain. Optimum configuration
                                          --   depends on clock frequency.
             );
     port (
           -- Clock and external reset inputs:
           Clk      : in  std_logic;
           ResetIn  : in  std_logic;
           -- Synchronized reset output:
           ResetOut : out std_logic
          );
   end component;
  
   signal reset : std_logic;
  
   signal enableCount : std_logic;
   signal enTen       : std_logic;
   signal enHundred   : std_logic;
  
   signal digitOne     : std_logic_vector (3 downto 0);
   signal digitTen     : std_logic_vector (3 downto 0);
   signal digitHundred : std_logic_vector (3 downto 0);
  
   signal memWe      : std_logic;
   signal memAddress : std_logic_vector (7 downto 0);
   signal memDataIn  : std_logic_vector (20 downto 0);
   signal memDataOut : std_logic_vector (20 downto 0);

begin

   -- Synchronize external reset signal:
   i_reset_adapt : reset_adapt
    port map (
          Clk      => Clk,
          ResetIn  => ResetX,
          ResetOut => reset
         );

   -- Counters for 0-255 count:
   i_cnt_1 : counter
    generic map ( COUNT_LENGTH => 10 )
    port map (
                Clk => Clk,
                Reset => reset,
                SyncReset => '0',
                EnableIn => enableCount,
                Count => digitOne,
                EnableOut => enTen
              );

   i_cnt_10 : counter
    generic map ( COUNT_LENGTH => 10 )
    port map (
                Clk => Clk,
                Reset => reset,
                SyncReset => '0',
                EnableIn => enTen,
                Count => digitTen,
                EnableOut => enHundred
              );

   i_cnt_100 : counter
    generic map ( COUNT_LENGTH => 3 )
    port map (
                Clk => Clk,
                Reset => reset,
                SyncReset => '0',
                EnableIn => enHundred,
                Count => digitHundred (1 downto 0),
                EnableOut => open
              );
             
   digitHundred (3 downto 2) <= "00";
   
   -- Decimal to seven segment converters:
   i_hex2led1 : hex2led
     port map (
               Hex => digitOne,
               Led => memDataIn (6 downto 0)              
              );
   i_hex2led10 : hex2led
     port map (
               Hex => digitTen,
               Led => memDataIn (13 downto 7)              
              );
   i_hex2led100 : hex2led
     port map (
               Hex => digitHundred,
               Led => memDataIn (20 downto 14)              
              );
   
   -- Display control: activate anodes and segments signals:
   i_display_ctrl: display_ctrl
     port map (
               Clk => Clk,
               Reset => reset,
               Digit0 => memDataOut (6 downto 0),
               Digit1 => memDataOut (13 downto 7),
               Digit2 => memDataOut (20 downto 14),
               Segments => Segments,
               Anodes => Anodes
              );
   
   -- RAM to store the seven segment values for the different numbers:
   i_ram : spram
     generic map (
              SIZE => 256,
              ADDR_WIDTH => 8,
              DATA_WIDTH => 21
             )
     port map (
               Clk      => Clk,
               We       => memWe,
               Address  => memAddress,
               DataIn   => memDataIn,
               DataOut  => memDataOut
              );   
   
   -- Control block: first store the seven segment values, then read RAM
   --  according to binary number representation in switches:
   i_ctrl : bin_reader_ctrl
     port map (
               Clk         => Clk,
               Reset       => reset,
               Switches    => Switches,
               MemWe       => memWe,
               MemAddress  => memAddress,
               EnableCount => enableCount
              ); 


end Behavioral;

