----------------------------------------------------------------------------------
-- Company: UAM 
-- Design Name: counter
-- Project Name: bin_reader
-- Description: 
--   Parameterizable counter. Being the value of the generic COUNT_LENGTH,
--   the counter counts from 0 to (COUNT_LENGTH-1).
--   The size of the output count port is derived automatically from the
--   generic.
-- Dependencies: 
--   bin_reader_pkg
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.bin_reader_pkg.all;   -- use project's package

-------------------------------------------------------------------------------
-- Entity

entity counter is
  generic (
           COUNT_LENGTH : integer := 100  -- Count 0 to (COUNT_LENGTH-1)
          );
  port (
        -- Inputs:
        Clk       : in std_logic;  -- Clock (rising edge)
        Reset     : in std_logic;  -- Async reset, active high
        SyncReset : in std_logic;  -- Sync reset, active high
        EnableIn  : in std_logic;  -- Clock enable. If 0 count is stopped
        -- Outputs:
        Count     : out std_logic_vector (log2_ceil(COUNT_LENGTH)-1 downto 0);
                                   -- Output count value
        EnableOut : out std_logic  -- '1' when Count is last value and EnableIn
                                   --   (combinational output)
       );
end counter;

-------------------------------------------------------------------------------
-- Architecture

architecture Behavioral of counter is

  -- Internal signal to implement the count:
  signal iCount     : std_logic_vector (log2_ceil(COUNT_LENGTH)-1 downto 0);
  -- Gets '1' with max count value:          
  signal maxReached : std_logic;
  
begin

  -- Set maxReached when final count value is reached:
  maxReached <= '1' when conv_integer(iCount) = (COUNT_LENGTH-1) else '0';

  -- Implement the counter (iCount). Reset with maxReached:
  process (Clk, Reset)
  begin
    if Reset = '1' then
      iCount <= (others => '0');
    elsif rising_edge (Clk) then
      if SyncReset = '1' then
        iCount <= (others => '0');
      elsif EnableIn = '1' then
        if maxReached = '1' then
          iCount <= (others => '0');
        else
          iCount <= iCount + 1;
        end if;
      end if;
    end if;
  end process;

  -- Assign outputs: Count and EnableOut (active when EnableIn and maxReached):
  Count <= iCount;
  EnableOut <= EnableIn and maxReached;  

end Behavioral;

