----------------------------------------------------------------------------------
-- Company: UAM 
-- Design Name: display_ctrl
-- Project Name: bin_reader
-- Description: 
--   Provides 7-segment coded, multiplexed output for a 3-digit display
--   based on 3x 7-segment input data.
--   Includes refresh anode cycling.
-- Dependencies: 
--   bin_reader_pkg
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.bin_reader_pkg.all;   -- use project's package

-------------------------------------------------------------------------------
-- Entity

entity display_ctrl is
  generic (
           REFRESH_NCYCLES : integer := 100000 -- Cycles between anode changes
          );
  port (
        Clk           : in  std_logic;
        Reset         : in  std_logic;
        Digit0        : in  std_logic_vector (6 downto 0);
        Digit1        : in  std_logic_vector (6 downto 0);
        Digit2        : in  std_logic_vector (6 downto 0);
        Segments      : out std_logic_vector (7 downto 0);
        Anodes        : out std_logic_vector (3 downto 0) 
       );        
end display_ctrl;

-------------------------------------------------------------------------------
-- Architecture

architecture Behavioral of display_ctrl is

  constant COUNT_WIDTH : integer := log2_ceil (REFRESH_NCYCLES);
  
  --  Refreshing interval counter:
  signal counter : std_logic_vector (COUNT_WIDTH-1 downto 0);
  --  Anodes signals (independent signal to be able to read Anodes output):
  signal anodesInt : std_logic_vector (3 downto 0);

begin

  -- Cyclic refresh for the 4 digits of the display 
  process (Clk, Reset)
  begin
    if Reset = '1' then
      counter  <= (others => '0');
      anodesInt <= "1110";
    elsif Clk'event and Clk = '1' then
      -- If interval reached, set next anode and clear counter,otherwise count:
      if counter = (REFRESH_NCYCLES-1) then
        counter  <= (others => '0');
        anodesInt <= anodesInt(2 downto 0) & anodesInt(3); -- rotate left
      else
        counter <= counter + 1;
      end if;
    end if;
  end process;
  
  Anodes <= anodesInt;

  -- Digit data selection according to active display anode:
  with anodesInt select
    Segments  <= '1' & Digit0    when "1110",
                 '1' & Digit1    when "1101",
                 '1' & Digit2    when "1011",
                 (others => '1') when others;
   
  
end Behavioral;

